Package: DGEAR
Type: Package
Title: Differential Gene Expression Analysis with R
Version: 0.1.4
Authors@R: c(person("Koushik", "Bardhan", role = c("aut","cre","ctb"),email="koushikbardhan2000@gmail.com", comment = c(ORCID = "0009-0002-8846-8347")),
            person("Chiranjib", "Sarkar", role = c("aut","ths"),email="cschiranjib9@gmail.com", comment = c(ORCID = "0000-0003-1536-7449")))
Author: Koushik Bardhan [aut, cre, ctb]
    (<https://orcid.org/0009-0002-8846-8347>),
  Chiranjib Sarkar [aut, ths] (<https://orcid.org/0000-0003-1536-7449>)
Maintainer: Koushik Bardhan <koushikbardhan2000@gmail.com>
Description: Analyses gene expression data derived from experiments to detect differentially expressed genes by employing the concept of majority voting with five different statistical models. It includes functions for differential expression analysis, significance testing, etc. It simplifies the process of uncovering meaningful patterns and trends within gene expression data, aiding researchers in downstream analysis. Boyer, R.S., Moore, J.S. (1991) <doi:10.1007/978-94-011-3488-0_5>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: DescTools
RoxygenNote: 7.3.1
Depends: R (>= 2.10)
Suggests: spelling, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-06-26 20:08:54 UTC; koush
Repository: CRAN
Date/Publication: 2024-06-26 20:20:02 UTC
Language: en-US
Built: R 4.6.0; ; 2026-04-25 12:54:37 UTC; unix

Package: DGEAR
Type: Package
Title: Differential Gene Expression Analysis with R
Version: 0.2.1
Authors@R: c(person("Koushik", "Bardhan", role = c("aut","cre","ctb"),
               email = "koushikbardhan2000@gmail.com",
               comment = c(ORCID = "0009-0002-8846-8347")),
             person("Chiranjib", "Sarkar", role = c("aut","ths"),
               email = "cschiranjib9@gmail.com",
               comment = c(ORCID = "0000-0003-1536-7449")))
Description: Analyses gene expression data derived from microarray experiments
    to detect differentially expressed genes (DEGs) by employing majority voting
    across five statistical models: Welch t-test, one-way ANOVA, Dunnett's test,
    Half's modified t-test, and the Wilcoxon-Mann-Whitney U-test.  Combined
    p-values are computed with Fisher's method.  Gene annotation is optional:
    users may supply a GEO SOFT annotation table or rely on row names directly.
    Boyer, R.S., Moore, J.S. (1991) <doi:10.1007/978-94-011-3488-0_5>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: DescTools, metapod
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0)
Suggests: spelling, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Language: en-US
Packaged: 2026-07-03 07:32:58 UTC; koush
Author: Koushik Bardhan [aut, cre, ctb] (ORCID:
    <https://orcid.org/0009-0002-8846-8347>),
  Chiranjib Sarkar [aut, ths] (ORCID:
    <https://orcid.org/0000-0003-1536-7449>)
Maintainer: Koushik Bardhan <koushikbardhan2000@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-03 08:50:35 UTC

# Generated by roxygen2: do not edit by hand

export(DGEAR)
export(perform_anova)
export(perform_dunnett_test)
export(perform_h_test)
export(perform_t_test)
export(perform_wilcox_test)
export(read_and_preprocess_data)
importFrom(DescTools,DunnettTest)
importFrom(stats,na.omit)
importFrom(stats,oneway.test)
importFrom(stats,p.adjust)
importFrom(stats,pt)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,wilcox.test)
importFrom(utils,read.delim)

# DGEAR

> **Differential Gene Expression Analysis with R**

The goal of DGEAR is to help researchers identify differentially expressed
genes (DEGs) from microarray gene expression data or RNA-seq count data in
the simplest way possible. DGEAR runs **five independent statistical tests**,
combines their results using a **majority voting** strategy, and reports DEGs
that are consistently flagged across methods.

---

## How it works

DGEAR applies five statistical tests to every gene in your expression matrix:

| Test | Function |
|------|----------|
| Welch two-sample t-test | `perform_t_test()` |
| One-way ANOVA | `perform_anova()` |
| Dunnett's test | `perform_dunnett_test()` |
| Half's modified t-test | `perform_h_test()` |
| Wilcoxon-Mann-Whitney U-test | `perform_wilcox_test()` |

Each test independently flags a gene as significant (BH-adjusted p ≤ α).
A gene is reported as a DEG when it is flagged by at least `votting_cutoff`
out of 5 tests. Combined p-values across all five tests are also computed
using Fisher's method (`metapod::parallelFisher`).

---

## Installation

Install the released version from CRAN:

``` r
install.packages("DGEAR")
```

Or install the development version from GitHub:

``` r
# install.packages("remotes")
remotes::install_github("koushikbardhan2000/DGEAR_0.2.1")
```

DGEAR depends on two packages. Install them once if needed:

``` r
# DescTools is on CRAN:
install.packages("DescTools")

# metapod is on Bioconductor:
if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")
BiocManager::install("metapod")
```

---

## Quick start

``` r
library(DGEAR)

# Load the built-in example dataset (10 genes × 20 samples)
# Columns 1–10 = control, columns 11–20 = experiment
# Genes 1–5 are true DEGs (expression ~13× higher in control)
data("gene_exp_data")
head(gene_exp_data)
```

### Run the full ensemble analysis

``` r
result <- DGEAR(
  dataframe      = gene_exp_data,
  con1           = 1,
  con2           = 10,
  exp1           = 11,
  exp2           = 20,
  alpha          = 0.05,
  votting_cutoff = 3        # gene must be flagged by at least 3/5 tests
)

# DEGs identified by majority voting
result$DEGs

# Full table: per-test BH FDRs, Fisher-combined FDR, log2FC, ensemble score
head(result$FDR_Table)

# Concise summary table
head(result$Results_Table)

# Raw output from each individual test
names(result$IndividualTests)   # "t_test" "anova" "dunnett" "half_t" "wilcoxon"
```

### Run individual tests

Each test function can also be called on its own and returns a `Table` and
a `DEGs` data.frame:

``` r
perform_t_test(gene_exp_data, con1 = 1, con2 = 10, exp1 = 11, exp2 = 20)
perform_anova(gene_exp_data, con1 = 1, con2 = 10, exp1 = 11, exp2 = 20)
perform_dunnett_test(gene_exp_data, con1 = 1, con2 = 10, exp1 = 11, exp2 = 20)
perform_h_test(gene_exp_data, con1 = 1, con2 = 10, exp1 = 11, exp2 = 20)
perform_wilcox_test(gene_exp_data, con1 = 1, con2 = 10, exp1 = 11, exp2 = 20)
```

### Using a GEO SOFT annotation file (optional)

Gene annotation is entirely optional. When no annotation is supplied, row
names are used as gene identifiers. To map probe IDs to gene symbols using
a GEO SOFT family file:

``` r
annot <- read.delim("GSExxxxx_family.soft")

result <- DGEAR(
  dataframe      = your_data,
  con1           = 1, con2 = 8,
  exp1           = 9, exp2 = 21,
  alpha          = 0.05,
  votting_cutoff = 3,
  annot_df       = annot        # <-- optional annotation table
)
```

---

## Data format

DGEAR expects a plain `data.frame` or `matrix` where:

- **Rows** are genes or probes
- **Columns** are samples
- **Control samples** occupy a contiguous block of columns (`con1` to `con2`)
- **Experiment samples** occupy a contiguous block of columns (`exp1` to `exp2`)

Raw intensity / count values are automatically log2-transformed when the data
appear to be on a linear scale (no manual transformation needed).

---

## Run the interactive demo

A fully annotated walkthrough script is bundled with the package:

``` r
source(system.file("examples", "DGEAR_demo.R", package = "DGEAR"))
```

---

## Function reference

| Function | Description |
|----------|-------------|
| `DGEAR()` | Main ensemble function — runs all five tests and returns DEGs |
| `perform_t_test()` | Welch two-sample t-test |
| `perform_anova()` | One-way ANOVA |
| `perform_dunnett_test()` | Dunnett's multiple comparison test |
| `perform_h_test()` | Half's modified t-test |
| `perform_wilcox_test()` | Wilcoxon-Mann-Whitney U-test |

---

## Authors

- **Koushik Bardhan** (author, maintainer) — [ORCID](https://orcid.org/0009-0002-8846-8347)
- **Chiranjib Sarkar** (author, thesis supervisor) — [ORCID](https://orcid.org/0000-0003-1536-7449)

## License

MIT © Koushik Bardhan

# DGEAR

<!-- badges: start -->
<!-- badges: end -->

The goal of DGEAR is to help the researchers find differentially expressed gene from microarray gene expression data or from the RNA seq count data the easiest way possible.

## Installation

You can install the development version of DGEAR like so:

``` r
# Simple installation:
install.packages("DGEAR")
```

## Library calling


``` r
library(DGEAR)
```
## Data format

DGEAR has it's own example data that can be accessible like so: 
``` r
# Data will be loaded with lazy loading and can be accessible when needed.
data("gene_exp_data")
head(gene_exp_data)
```
## How it works
Here's an example code to run the example dataset.

``` r
library(DGEAR)
data("gene_exp_data")
DGEAR(dataframe = gene_exp_data, con1 = 1, con2 = 10,
  exp1 = 11, exp2 = 20, alpha = 0.05, votting_cutoff = 2)
```
Try to find the DEGs from the example dataset with all the default arguments and understand it's working. 

#' Welch Two-Sample t-Test for Differential Gene Expression
#'
#' Performs an independent two-sample Welch t-test for every gene (row) in the
#' expression matrix, applies Benjamini-Hochberg (BH) correction, and returns
#' a results table together with the list of significant DEGs.
#'
#' @param dataframe A numeric matrix or data.frame of gene expression values
#'   (rows = genes, columns = samples).  Values are automatically log2-
#'   transformed when they appear to be on a linear / intensity scale.
#' @param con1 Integer. Index of the first control column.
#' @param con2 Integer. Index of the last control column.
#' @param exp1 Integer. Index of the first experiment column.
#' @param exp2 Integer. Index of the last experiment column.
#' @param alpha Numeric significance threshold for BH-adjusted p-values
#'   (default \code{0.05}).
#' @param annot_df Optional annotation data.frame with columns \code{ID} and
#'   \code{Gene.Symbol} (or \code{Gene.symbol}).  When \code{NULL} (default)
#'   row names of \code{dataframe} are used as gene identifiers.
#'
#' @return A named list:
#'   \describe{
#'     \item{Table}{Data.frame with columns \code{G_Symbol}, \code{log2FC},
#'       \code{statistic.t}, \code{p.value}, \code{BH}, \code{fdr}.}
#'     \item{DEGs}{Data.frame of gene identifiers whose BH-adjusted p-value
#'       is \eqn{\le} \code{alpha}.}
#'   }
#'
#' @export
#' @importFrom stats t.test p.adjust na.omit quantile
#' @examples
#' library(DGEAR)
#' data("gene_exp_data")
#' result <- perform_t_test(dataframe = gene_exp_data,
#'                          con1 = 1, con2 = 10,
#'                          exp1 = 11, exp2 = 20)
#' head(result$Table)
#' result$DEGs
perform_t_test <- function(dataframe, con1, con2, exp1, exp2,
                           alpha = 0.05, annot_df = NULL) {
  prep       <- .prepare_data(dataframe, con1, con2, exp1, exp2)
  G_Symbol   <- .resolve_symbols(prep$dataframe, annot_df)
  con        <- prep$con
  exp        <- prep$exp
  log2FC     <- prep$log2FC
  
  t_results <- t(sapply(seq_len(nrow(con)), function(i) {
    res <- t.test(as.numeric(con[i, ]), as.numeric(exp[i, ]))
    c(statistic.t = res$statistic, p.value = res$p.value)
  }))
  
  t_stat         <- data.frame(G_Symbol, log2FC, t_results,
                               stringsAsFactors = FALSE)
  t_stat$BH      <- p.adjust(t_stat$p.value, method = "BH")
  t_stat$fdr     <- as.integer(t_stat$BH <= alpha)
  
  DEGs <- data.frame(DEGs = na.omit(t_stat$G_Symbol[t_stat$fdr == 1L]),
                     stringsAsFactors = FALSE)
  list(Table = t_stat, DEGs = DEGs)
}

#' Dunnett's Test for Differential Gene Expression
#'
#' Performs Dunnett's multiple comparison test (control vs. treatment) for every
#' gene (row) in the expression matrix using \code{\link[DescTools]{DunnettTest}},
#' applies BH correction, and returns a results table with the list of significant
#' DEGs.  Genes with insufficient variance, missing values, or other numerical
#' issues are skipped gracefully and receive \code{NA} p-values.
#'
#' @inheritParams perform_t_test
#'
#' @return A named list:
#'   \describe{
#'     \item{Table}{Data.frame with columns \code{G_Symbol}, \code{log2FC},
#'       \code{p.value}, \code{BH}, \code{fdr}.}
#'     \item{DEGs}{Data.frame of significant gene identifiers.}
#'   }
#'
#' @export
#' @importFrom DescTools DunnettTest
#' @importFrom stats p.adjust na.omit quantile var
#' @examples
#' library(DGEAR)
#' data("gene_exp_data")
#' result <- perform_dunnett_test(dataframe = gene_exp_data,
#'                                con1 = 1, con2 = 10,
#'                                exp1 = 11, exp2 = 20)
#' head(result$Table)
#' result$DEGs
perform_dunnett_test <- function(dataframe, con1, con2, exp1, exp2,
                                 alpha = 0.05, annot_df = NULL) {
  prep     <- .prepare_data(dataframe, con1, con2, exp1, exp2)
  G_Symbol <- .resolve_symbols(prep$dataframe, annot_df)
  df_work  <- prep$dataframe
  log2FC   <- prep$log2FC
  
  group <- factor(c(rep("con", con2 - con1 + 1L),
                    rep("tre", exp2 - exp1 + 1L)))
  
  df_t <- t(as.matrix(df_work))  # samples × genes
  
  p.values <- vapply(seq_len(ncol(df_t)), function(i) {
    x <- as.numeric(df_t[, i])
    # Defensive: skip genes with NA, Inf, zero-variance, or too few finite obs
    if (any(!is.finite(x)) || var(x, na.rm = TRUE) == 0 ||
        sum(is.finite(x)) < length(levels(group))) {
      return(NA_real_)
    }
    tryCatch({
      d <- DescTools::DunnettTest(x, group)
      as.numeric(d$con[, 4L])
    }, error = function(e) NA_real_,
    warning = function(w) {
      suppressWarnings({
        d <- DescTools::DunnettTest(x, group)
        as.numeric(d$con[, 4L])
      })
    })
  }, numeric(1L))
  
  d_stat         <- data.frame(G_Symbol, log2FC, p.value = p.values,
                               stringsAsFactors = FALSE)
  d_stat$BH      <- p.adjust(d_stat$p.value, method = "BH")
  d_stat$fdr     <- as.integer(!is.na(d_stat$BH) & d_stat$BH <= alpha)
  
  DEGs <- data.frame(DEGs = na.omit(d_stat$G_Symbol[d_stat$fdr == 1L]),
                     stringsAsFactors = FALSE)
  list(Table = d_stat, DEGs = DEGs)
}

#' One-Way ANOVA Test for Differential Gene Expression
#'
#' Performs a one-way ANOVA (Welch correction via \code{\link[stats]{oneway.test}})
#' for every gene (row) in the expression matrix, applies BH correction, and
#' returns a results table together with the list of significant DEGs.
#'
#' @inheritParams perform_t_test
#'
#' @return A named list:
#'   \describe{
#'     \item{Table}{Data.frame with columns \code{G_Symbol}, \code{log2FC},
#'       \code{statistic.F}, \code{p.value}, \code{BH}, \code{fdr}.}
#'     \item{DEGs}{Data.frame of significant gene identifiers.}
#'   }
#'
#' @export
#' @importFrom stats oneway.test p.adjust na.omit quantile
#' @examples
#' library(DGEAR)
#' data("gene_exp_data")
#' result <- perform_anova(dataframe = gene_exp_data,
#'                         con1 = 1, con2 = 10,
#'                         exp1 = 11, exp2 = 20)
#' head(result$Table)
#' result$DEGs
perform_anova <- function(dataframe, con1, con2, exp1, exp2,
                          alpha = 0.05, annot_df = NULL) {
  prep     <- .prepare_data(dataframe, con1, con2, exp1, exp2)
  G_Symbol <- .resolve_symbols(prep$dataframe, annot_df)
  log2FC   <- prep$log2FC
  df_t     <- t(as.matrix(prep$dataframe))   # samples × genes
  
  group <- factor(c(rep("con", con2 - con1 + 1L),
                    rep("tre", exp2 - exp1 + 1L)))
  
  o_results <- t(sapply(seq_len(ncol(df_t)), function(i) {
    res <- oneway.test(df_t[, i] ~ group)
    c(statistic.F = res$statistic, p.value = res$p.value)
  }))
  
  o_stat         <- data.frame(G_Symbol, log2FC, o_results,
                               stringsAsFactors = FALSE)
  o_stat$BH      <- p.adjust(o_stat$p.value, method = "BH")
  o_stat$fdr     <- as.integer(o_stat$BH <= alpha)
  
  DEGs <- data.frame(DEGs = na.omit(o_stat$G_Symbol[o_stat$fdr == 1L]),
                     stringsAsFactors = FALSE)
  list(Table = o_stat, DEGs = DEGs)
}

# ---- Internal helpers (not exported) ----

#' Resolve gene symbols from row names or an optional annotation table
#'
#' @param dataframe Gene expression data.frame (rows = probes/genes).
#' @param annot_df  Optional annotation data.frame with columns \code{ID} and
#'   one of \code{Gene.Symbol} / \code{Gene.symbol}.  When \code{NULL} (the
#'   default) row names of \code{dataframe} are used as gene identifiers.
#' @return A character vector of gene symbols the same length as
#'   \code{nrow(dataframe)}.
#' @keywords internal
.resolve_symbols <- function(dataframe, annot_df = NULL) {
  if (is.null(annot_df)) {
    return(rownames(dataframe))
  }
  # Detect symbol column
  sym_col <- if ("Gene.Symbol" %in% colnames(annot_df)) {
    "Gene.Symbol"
  } else if ("Gene.symbol" %in% colnames(annot_df)) {
    "Gene.symbol"
  } else {
    warning("Could not find 'Gene.Symbol' or 'Gene.symbol' in annot_df; ",
            "using row names instead.")
    return(rownames(dataframe))
  }
  annot_df <- annot_df[!duplicated(annot_df$ID), ]
  ID <- data.frame(GeneID = rownames(dataframe), stringsAsFactors = FALSE)
  merged <- merge(ID, annot_df, by.x = "GeneID", by.y = "ID", all.x = TRUE,
                  sort = FALSE)
  # Keep original order
  merged <- merged[match(ID$GeneID, merged$GeneID), ]
  raw_sym <- as.character(merged[[sym_col]])
  # Take the first symbol when multiple are separated by " /// "
  sapply(strsplit(raw_sym, " /// "), function(x) x[[1]])
}

#' Apply log2 normalisation if the data appear to be on a linear scale
#'
#' @param dataframe Numeric data.frame.
#' @param con1,con2,exp1,exp2 Column indices.
#' @return A list with \code{dataframe} (possibly log2-transformed),
#'   \code{con}, \code{exp}, \code{con_m}, \code{exp_m}, \code{log2FC},
#'   and logical \code{log_transformed}.
#' @keywords internal
.prepare_data <- function(dataframe, con1, con2, exp1, exp2) {
  con <- dataframe[, con1:con2, drop = FALSE]
  exp <- dataframe[, exp1:exp2, drop = FALSE]
  con_m <- rowMeans(con, na.rm = TRUE)
  exp_m <- rowMeans(exp, na.rm = TRUE)
  
  xm2 <- as.numeric(quantile(as.matrix(dataframe),
                             c(0, 0.25, 0.5, 0.75, 0.99, 1),
                             na.rm = TRUE))
  LogC <- (xm2[5] > 100) || (xm2[6] - xm2[1] > 50 && xm2[2] > 0)
  
  if (LogC) {
    dataframe <- log2(dataframe + 1)
    con   <- dataframe[, con1:con2, drop = FALSE]
    exp   <- dataframe[, exp1:exp2, drop = FALSE]
    con_m <- rowMeans(con, na.rm = TRUE)
    exp_m <- rowMeans(exp, na.rm = TRUE)
  }
  log2FC <- exp_m - con_m
  list(dataframe = dataframe, con = con, exp = exp,
       con_m = con_m, exp_m = exp_m, log2FC = log2FC,
       log_transformed = LogC)
}

#' Wilcoxon-Mann-Whitney U-Test for Differential Gene Expression
#'
#' Performs the Wilcoxon rank-sum (Mann-Whitney U) test for every gene (row)
#' in the expression matrix, applies BH correction, and returns a results
#' table together with the list of significant DEGs.
#'
#' @inheritParams perform_t_test
#'
#' @return A named list:
#'   \describe{
#'     \item{Table}{Data.frame with columns \code{G_Symbol}, \code{log2FC},
#'       \code{statistic.W}, \code{p.value}, \code{BH}, \code{fdr}.}
#'     \item{DEGs}{Data.frame of significant gene identifiers.}
#'   }
#'
#' @export
#' @importFrom stats wilcox.test p.adjust na.omit quantile
#' @examples
#' library(DGEAR)
#' data("gene_exp_data")
#' result <- perform_wilcox_test(dataframe = gene_exp_data,
#'                               con1 = 1, con2 = 10,
#'                               exp1 = 11, exp2 = 20)
#' head(result$Table)
#' result$DEGs
perform_wilcox_test <- function(dataframe, con1, con2, exp1, exp2,
                                alpha = 0.05, annot_df = NULL) {
  prep     <- .prepare_data(dataframe, con1, con2, exp1, exp2)
  G_Symbol <- .resolve_symbols(prep$dataframe, annot_df)
  con      <- prep$con
  exp      <- prep$exp
  log2FC   <- prep$log2FC
  
  u_results <- t(sapply(seq_len(nrow(con)), function(i) {
    res <- suppressWarnings(
      wilcox.test(as.numeric(con[i, ]), as.numeric(exp[i, ]), paired = FALSE)
    )
    c(statistic.W = res$statistic, p.value = res$p.value)
  }))
  
  u_stat         <- data.frame(G_Symbol, log2FC, u_results,
                               stringsAsFactors = FALSE)
  u_stat$BH      <- p.adjust(u_stat$p.value, method = "BH")
  u_stat$fdr     <- as.integer(u_stat$BH <= alpha)
  
  DEGs <- data.frame(DEGs = na.omit(u_stat$G_Symbol[u_stat$fdr == 1L]),
                     stringsAsFactors = FALSE)
  list(Table = u_stat, DEGs = DEGs)
}

#' Half's Modified t-Test for Differential Gene Expression
#'
#' Computes a modified t-statistic (sometimes called "Half's t-test") that uses
#' only the control standard deviation in its denominator, applies BH correction,
#' and returns a results table together with the list of significant DEGs.
#'
#' @inheritParams perform_t_test
#'
#' @return A named list:
#'   \describe{
#'     \item{Table}{Data.frame with columns \code{G_Symbol}, \code{log2FC},
#'       \code{statistic}, \code{p.value}, \code{BH}, \code{fdr}.}
#'     \item{DEGs}{Data.frame of significant gene identifiers.}
#'   }
#'
#' @export
#' @importFrom stats sd pt p.adjust na.omit quantile
#' @examples
#' library(DGEAR)
#' data("gene_exp_data")
#' result <- perform_h_test(dataframe = gene_exp_data,
#'                          con1 = 1, con2 = 10,
#'                          exp1 = 11, exp2 = 20)
#' head(result$Table)
#' result$DEGs
perform_h_test <- function(dataframe, con1, con2, exp1, exp2,
                           alpha = 0.05, annot_df = NULL) {
  prep     <- .prepare_data(dataframe, con1, con2, exp1, exp2)
  G_Symbol <- .resolve_symbols(prep$dataframe, annot_df)
  con      <- prep$con
  exp      <- prep$exp
  log2FC   <- prep$log2FC
  
  s0          <- apply(con, 1, sd, na.rm = TRUE)
  sample_size <- sqrt((1 / ncol(exp)) + (1 / ncol(con)))
  statistic   <- log2FC / (s0 * sample_size)
  
  half_t_stat        <- data.frame(G_Symbol, log2FC, statistic,
                                   stringsAsFactors = FALSE)
  half_t_stat$p.value <- 2 * pt(abs(statistic), df = nrow(con) - 1L,
                                lower.tail = FALSE)
  half_t_stat$BH     <- p.adjust(half_t_stat$p.value, method = "BH")
  half_t_stat$fdr    <- as.integer(half_t_stat$BH <= alpha)
  
  DEGs <- data.frame(
    DEGs = na.omit(half_t_stat$G_Symbol[half_t_stat$fdr == 1L]),
    stringsAsFactors = FALSE)
  list(Table = half_t_stat, DEGs = DEGs)
}

#' Differential Gene Expression Analysis with R
#'
#' @description
#' Main orchestration function that runs five statistical tests (Welch t-test,
#' one-way ANOVA, Dunnett's test, Half's modified t-test, and Wilcoxon-Mann-Whitney
#' U-test) on a gene expression matrix, combines their BH-adjusted p-values with
#' Fisher's combined probability method, and identifies differentially expressed
#' genes (DEGs) by majority voting.
#'
#' @param dataframe A numeric matrix or data.frame of gene expression values
#'   (rows = genes, columns = samples).  Raw intensity / count values are
#'   automatically log2-transformed when they appear to be on a linear scale.
#' @param con1 Integer. Index of the first control column.
#' @param con2 Integer. Index of the last control column.
#' @param exp1 Integer. Index of the first experiment column.
#' @param exp2 Integer. Index of the last experiment column.
#' @param alpha Numeric significance threshold for BH-adjusted p-values
#'   (default \code{0.05}).
#' @param votting_cutoff Integer. Minimum number of tests (out of 5) that must
#'   independently declare a gene significant for it to be included in the
#'   majority-vote DEG list (default \code{3}).  Must be between 1 and 5.
#' @param annot_df Optional annotation data.frame with columns \code{ID} and
#'   \code{Gene.Symbol} (or \code{Gene.symbol}) that maps probe/row identifiers
#'   to gene symbols.  When \code{NULL} (default) row names of \code{dataframe}
#'   are used directly as gene identifiers.  Typical source: a GEO SOFT family
#'   file parsed with \code{read.delim()}.
#'
#' @details
#' The function internally calls:
#' \itemize{
#'   \item \code{\link{perform_t_test}} — Welch two-sample t-test
#'   \item \code{\link{perform_anova}} — one-way ANOVA
#'   \item \code{\link{perform_dunnett_test}} — Dunnett's test
#'   \item \code{\link{perform_h_test}} — Half's modified t-test
#'   \item \code{\link{perform_wilcox_test}} — Wilcoxon rank-sum test
#' }
#' Each test independently assigns an FDR flag (1 = significant, 0 = not).
#' The five flags are summed per gene; genes whose sum meets or exceeds
#' \code{votting_cutoff} are reported as DEGs (majority voting, Boyer & Moore
#' 1991).  Combined p-values across all five tests are computed with Fisher's
#' method via \code{\link[metapod]{parallelFisher}}.
#'
#' Annotation via \code{annot_df} is entirely optional.  When supplied, the
#' first gene symbol listed for each probe (delimited by \code{ /// }) is used.
#' When absent, row names serve as identifiers, making the function fully
#' self-contained without GEO annotation files.
#'
#' @return A named list with four elements:
#'   \describe{
#'     \item{\code{DEGs}}{Data.frame of gene identifiers that passed majority
#'       voting.}
#'     \item{\code{FDR_Table}}{Wide data.frame with BH-adjusted p-values from
#'       every test, the Fisher-combined FDR, the ensemble voting score, and
#'       log2 fold change for every gene.}
#'     \item{\code{Results_Table}}{Concise data.frame with \code{G_Symbol},
#'       \code{CombineFDR}, \code{log2FC}, and \code{Ensemble} score.}
#'     \item{\code{IndividualTests}}{Named list of the raw output from each
#'       of the five test functions (each containing a \code{Table} and a
#'       \code{DEGs} element).}
#'   }
#'
#' @references
#' Boyer, R.S. and Moore, J.S. (1991). MJRTY — A Fast Majority Vote Algorithm.
#' In \emph{Automated Reasoning: Essays in Honor of Woody Bledsoe}, pp. 105–117.
#' Springer, Dordrecht. \doi{10.1007/978-94-011-3488-0_5}
#'
#' @export
#' @importFrom stats quantile
#' @importFrom metapod parallelFisher
#' @examples
#' library(DGEAR)
#' data("gene_exp_data")
#'
#' ## Basic usage — no annotation file needed
#' result <- DGEAR(dataframe    = gene_exp_data,
#'                 con1         = 1,
#'                 con2         = 10,
#'                 exp1         = 11,
#'                 exp2         = 20,
#'                 alpha        = 0.05,
#'                 votting_cutoff = 2)
#' result$DEGs
#' head(result$FDR_Table)
#'
#' ## With an optional annotation data.frame (GEO SOFT format)
#' ## annot <- read.delim("GSExxxxx_family.soft")
#' ## result <- DGEAR(dataframe = gene_exp_data,
#' ##                 con1 = 1, con2 = 10, exp1 = 11, exp2 = 20,
#' ##                 annot_df = annot)
DGEAR <- function(dataframe, con1, con2, exp1, exp2,
                  alpha = 0.05, votting_cutoff = 3,
                  annot_df = NULL) {
  
  # ---- Validate inputs -------------------------------------------------------
  if (!is.data.frame(dataframe) && !is.matrix(dataframe))
    stop("'dataframe' must be a matrix or data.frame.")
  if (con1 < 1 || con2 > ncol(dataframe) || con1 > con2)
    stop("Invalid control column range (con1:con2).")
  if (exp1 < 1 || exp2 > ncol(dataframe) || exp1 > exp2)
    stop("Invalid experiment column range (exp1:exp2).")
  if (alpha <= 0 || alpha >= 1)
    stop("'alpha' must be strictly between 0 and 1.")
  if (!votting_cutoff %in% 1:5)
    stop("'votting_cutoff' must be an integer between 1 and 5.")
  
  # ---- Run the five statistical tests ----------------------------------------
  message("Running Welch t-test ...")
  t_res  <- perform_t_test(dataframe, con1, con2, exp1, exp2, alpha, annot_df)
  
  message("Running one-way ANOVA ...")
  o_res  <- perform_anova(dataframe, con1, con2, exp1, exp2, alpha, annot_df)
  
  message("Running Dunnett's test ...")
  d_res  <- perform_dunnett_test(dataframe, con1, con2, exp1, exp2, alpha, annot_df)
  
  message("Running Half's t-test ...")
  h_res  <- perform_h_test(dataframe, con1, con2, exp1, exp2, alpha, annot_df)
  
  message("Running Wilcoxon rank-sum test ...")
  u_res  <- perform_wilcox_test(dataframe, con1, con2, exp1, exp2, alpha, annot_df)
  
  # ---- Retrieve shared quantities from the first result ----------------------
  G_Symbol <- t_res$Table$G_Symbol
  log2FC   <- t_res$Table$log2FC
  
  # ---- Ensemble / majority voting --------------------------------------------
  fdr_mat <- data.frame(
    T.FDR = t_res$Table$fdr,
    O.FDR = o_res$Table$fdr,
    D.FDR = d_res$Table$fdr,
    H.FDR = h_res$Table$fdr,
    W.FDR = u_res$Table$fdr
  )
  fdr_mat[is.na(fdr_mat)] <- 0L
  ensemble_score <- rowSums(fdr_mat)
  
  DEGs <- data.frame(
    DEGs = G_Symbol[ensemble_score >= votting_cutoff],
    stringsAsFactors = FALSE
  )
  
  # ---- Combined p-value (Fisher's method via metapod) -----------------------
  P_list <- list(
    P1 = t_res$Table$BH,
    P2 = o_res$Table$BH,
    P3 = d_res$Table$BH,
    P4 = h_res$Table$BH,
    P5 = u_res$Table$BH
  )
  # Replace NA BH values with 1 before combining (conservative)
  P_list <- lapply(P_list, function(p) { p[is.na(p)] <- 1; p })
  CombineFDR <- metapod::parallelFisher(P_list)$p.value
  
  # ---- Summary tables --------------------------------------------------------
  FDR_Table <- data.frame(
    G_Symbol   = G_Symbol,
    log2FC     = log2FC,
    T.FDR      = t_res$Table$BH,
    O.FDR      = o_res$Table$BH,
    D.FDR      = d_res$Table$BH,
    H.FDR      = h_res$Table$BH,
    W.FDR      = u_res$Table$BH,
    CombineFDR = CombineFDR,
    Ensemble   = ensemble_score,
    stringsAsFactors = FALSE
  )
  
  Results_Table <- data.frame(
    G_Symbol   = G_Symbol,
    CombineFDR = CombineFDR,
    log2FC     = log2FC,
    Ensemble   = ensemble_score,
    stringsAsFactors = FALSE
  )
  
  message(sprintf("Done. %d DEGs identified at votting_cutoff = %d.",
                  nrow(DEGs), votting_cutoff))
  
  list(
    DEGs          = DEGs,
    FDR_Table     = FDR_Table,
    Results_Table = Results_Table,
    IndividualTests = list(
      t_test   = t_res,
      anova    = o_res,
      dunnett  = d_res,
      half_t   = h_res,
      wilcoxon = u_res
    )
  )
}

#' A dataset containing gene expression data
#'
#' This dataset contains statistically simulated gene expression data for ease of exercise.
#'
#' @docType data
#' @name gene_exp_data
#' @format A data frame with 10 rows and 20 columns, the columns represents samples, say first 10 columns 1 to 10 being control and 11 to 20 being experiment. Whereas, the rows of the dataset contains genes. First 5 out of 10 genes, gene1-gene5 are the true DEGs as the expression values for the first 10 samples are ~13 times higher than the rest. 
#' @examples
#' # Data will be loaded with lazy loading and can be accessible when needed.
#' data("gene_exp_data")
#' head(gene_exp_data)
"gene_exp_data"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.prepare_data}
\alias{.prepare_data}
\title{Apply log2 normalisation if the data appear to be on a linear scale}
\usage{
.prepare_data(dataframe, con1, con2, exp1, exp2)
}
\arguments{
\item{dataframe}{Numeric data.frame.}

\item{con1, con2, exp1, exp2}{Column indices.}
}
\value{
A list with \code{dataframe} (possibly log2-transformed),
  \code{con}, \code{exp}, \code{con_m}, \code{exp_m}, \code{log2FC},
  and logical \code{log_transformed}.
}
\description{
Apply log2 normalisation if the data appear to be on a linear scale
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.resolve_symbols}
\alias{.resolve_symbols}
\title{Resolve gene symbols from row names or an optional annotation table}
\usage{
.resolve_symbols(dataframe, annot_df = NULL)
}
\arguments{
\item{dataframe}{Gene expression data.frame (rows = probes/genes).}

\item{annot_df}{Optional annotation data.frame with columns \code{ID} and
one of \code{Gene.Symbol} / \code{Gene.symbol}.  When \code{NULL} (the
default) row names of \code{dataframe} are used as gene identifiers.}
}
\value{
A character vector of gene symbols the same length as
  \code{nrow(dataframe)}.
}
\description{
Resolve gene symbols from row names or an optional annotation table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene_exp_data.R
\docType{data}
\name{gene_exp_data}
\alias{gene_exp_data}
\title{A dataset containing gene expression data}
\format{
A data frame with 10 rows and 20 columns, the columns represents samples, say first 10 columns 1 to 10 being control and 11 to 20 being experiment. Whereas, the rows of the dataset contains genes. First 5 out of 10 genes, gene1-gene5 are the true DEGs as the expression values for the first 10 samples are ~13 times higher than the rest.
}
\usage{
gene_exp_data
}
\description{
This dataset contains statistically simulated gene expression data for ease of exercise.
}
\examples{
# Data will be loaded with lazy loading and can be accessible when needed.
data("gene_exp_data")
head(gene_exp_data)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DGEAR.R
\name{DGEAR}
\alias{DGEAR}
\title{Differential Gene Expression Analysis with R}
\usage{
DGEAR(
  dataframe,
  con1,
  con2,
  exp1,
  exp2,
  alpha = 0.05,
  votting_cutoff = 3,
  annot_df = NULL
)
}
\arguments{
\item{dataframe}{A numeric matrix or data.frame of gene expression values
(rows = genes, columns = samples).  Raw intensity / count values are
automatically log2-transformed when they appear to be on a linear scale.}

\item{con1}{Integer. Index of the first control column.}

\item{con2}{Integer. Index of the last control column.}

\item{exp1}{Integer. Index of the first experiment column.}

\item{exp2}{Integer. Index of the last experiment column.}

\item{alpha}{Numeric significance threshold for BH-adjusted p-values
(default \code{0.05}).}

\item{votting_cutoff}{Integer. Minimum number of tests (out of 5) that must
independently declare a gene significant for it to be included in the
majority-vote DEG list (default \code{3}).  Must be between 1 and 5.}

\item{annot_df}{Optional annotation data.frame with columns \code{ID} and
\code{Gene.Symbol} (or \code{Gene.symbol}) that maps probe/row identifiers
to gene symbols.  When \code{NULL} (default) row names of \code{dataframe}
are used directly as gene identifiers.  Typical source: a GEO SOFT family
file parsed with \code{read.delim()}.}
}
\value{
A named list with four elements:
  \describe{
    \item{\code{DEGs}}{Data.frame of gene identifiers that passed majority
      voting.}
    \item{\code{FDR_Table}}{Wide data.frame with BH-adjusted p-values from
      every test, the Fisher-combined FDR, the ensemble voting score, and
      log2 fold change for every gene.}
    \item{\code{Results_Table}}{Concise data.frame with \code{G_Symbol},
      \code{CombineFDR}, \code{log2FC}, and \code{Ensemble} score.}
    \item{\code{IndividualTests}}{Named list of the raw output from each
      of the five test functions (each containing a \code{Table} and a
      \code{DEGs} element).}
  }
}
\description{
Main orchestration function that runs five statistical tests (Welch t-test,
one-way ANOVA, Dunnett's test, Half's modified t-test, and Wilcoxon-Mann-Whitney
U-test) on a gene expression matrix, combines their BH-adjusted p-values with
Fisher's combined probability method, and identifies differentially expressed
genes (DEGs) by majority voting.
}
\details{
The function internally calls:
\itemize{
  \item \code{\link{perform_t_test}} — Welch two-sample t-test
  \item \code{\link{perform_anova}} — one-way ANOVA
  \item \code{\link{perform_dunnett_test}} — Dunnett's test
  \item \code{\link{perform_h_test}} — Half's modified t-test
  \item \code{\link{perform_wilcox_test}} — Wilcoxon rank-sum test
}
Each test independently assigns an FDR flag (1 = significant, 0 = not).
The five flags are summed per gene; genes whose sum meets or exceeds
\code{votting_cutoff} are reported as DEGs (majority voting, Boyer & Moore
1991).  Combined p-values across all five tests are computed with Fisher's
method via \code{\link[metapod]{parallelFisher}}.

Annotation via \code{annot_df} is entirely optional.  When supplied, the
first gene symbol listed for each probe (delimited by \code{ /// }) is used.
When absent, row names serve as identifiers, making the function fully
self-contained without GEO annotation files.
}
\examples{
library(DGEAR)
data("gene_exp_data")

## Basic usage — no annotation file needed
result <- DGEAR(dataframe    = gene_exp_data,
                con1         = 1,
                con2         = 10,
                exp1         = 11,
                exp2         = 20,
                alpha        = 0.05,
                votting_cutoff = 2)
result$DEGs
head(result$FDR_Table)

## With an optional annotation data.frame (GEO SOFT format)
## annot <- read.delim("GSExxxxx_family.soft")
## result <- DGEAR(dataframe = gene_exp_data,
##                 con1 = 1, con2 = 10, exp1 = 11, exp2 = 20,
##                 annot_df = annot)
}
\references{
Boyer, R.S. and Moore, J.S. (1991). MJRTY — A Fast Majority Vote Algorithm.
In \emph{Automated Reasoning: Essays in Honor of Woody Bledsoe}, pp. 105–117.
Springer, Dordrecht. \doi{10.1007/978-94-011-3488-0_5}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/D-Test.R
\name{perform_dunnett_test}
\alias{perform_dunnett_test}
\title{Dunnett's Test for Differential Gene Expression}
\usage{
perform_dunnett_test(
  dataframe,
  con1,
  con2,
  exp1,
  exp2,
  alpha = 0.05,
  annot_df = NULL
)
}
\arguments{
\item{dataframe}{A numeric matrix or data.frame of gene expression values
(rows = genes, columns = samples).  Values are automatically log2-
transformed when they appear to be on a linear / intensity scale.}

\item{con1}{Integer. Index of the first control column.}

\item{con2}{Integer. Index of the last control column.}

\item{exp1}{Integer. Index of the first experiment column.}

\item{exp2}{Integer. Index of the last experiment column.}

\item{alpha}{Numeric significance threshold for BH-adjusted p-values
(default \code{0.05}).}

\item{annot_df}{Optional annotation data.frame with columns \code{ID} and
\code{Gene.Symbol} (or \code{Gene.symbol}).  When \code{NULL} (default)
row names of \code{dataframe} are used as gene identifiers.}
}
\value{
A named list:
  \describe{
    \item{Table}{Data.frame with columns \code{G_Symbol}, \code{log2FC},
      \code{p.value}, \code{BH}, \code{fdr}.}
    \item{DEGs}{Data.frame of significant gene identifiers.}
  }
}
\description{
Performs Dunnett's multiple comparison test (control vs. treatment) for every
gene (row) in the expression matrix using \code{\link[DescTools]{DunnettTest}},
applies BH correction, and returns a results table with the list of significant
DEGs.  Genes with insufficient variance, missing values, or other numerical
issues are skipped gracefully and receive \code{NA} p-values.
}
\examples{
library(DGEAR)
data("gene_exp_data")
result <- perform_dunnett_test(dataframe = gene_exp_data,
                               con1 = 1, con2 = 10,
                               exp1 = 11, exp2 = 20)
head(result$Table)
result$DEGs
}
